<!---
Copyright 2007 Ed Tabara

Licensed under the Apache License, Version 2.0 (the &quot;License&quot;);
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an &quot;AS IS&quot; BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
--->
<!---
Creator: Eduard Tabara
Email: 1smartsolution@gmail.com
Version: 1
--->
<!---
Modified by: Steven Durette
Email: sdurette@prodigy.net
Version: 2
Notes:
1) Added options for XLSX (Office 2007) files if the POI is the right version.
2) Added getRow function. This function will get a row or create it if it doesn't already exist.
3) Added getCell function. This function will get a cell or create it if it doesn't already exist.
4) Changed multiple functions to use getRow and getCell functions.
5) Added AddImage function to place an image in the spreadsheet.
6) Added AddJPGImage function to put a JPEG image in the spreadsheet.
7) Added AddPNGImage function to put a PNG image in the spreadsheet.
8) Added createBookVar to allow for streaming workbook to the browser without creating a file.
9) Updated create region to allow creation of regions in xlsx workbooks.
--->
<cfcomponent displayname="Excel" hint="I will help you to work with Excel">
	<cffunction name="init" output="No" returntype="any" hint="I initialize the object">
		<!--- Available options are XLS and XLSX --->
		<cfargument type="string" name="wbType" default="XLS" />
		<cfset var newWorkbook = "" />

		<cfif arguments.wbType is "XLSX">
			<cftry>
				<cfset newWorkbook = createObject("java", "org.apache.poi.xssf.usermodel.XSSFWorkbook").init() />

				<cfcatch type="object">
					<cfset newWorkbook = createObject("java","org.apache.poi.hssf.usermodel.HSSFWorkbook").init() />
				</cfcatch>
			</cftry>
		<cfelse>
			<cfset newWorkbook = createObject("java", "org.apache.poi.hssf.usermodel.HSSFWorkbook").init() />
		</cfif>
		<cfreturn newWorkbook />
	</cffunction>

	<cffunction name="close" output="No" returntype="any" hint="I destroy the object">
		<cfargument type="any" name="vr" default="fileOut">
		<cfreturn vr.close()>
	</cffunction>

	<cffunction name="createBook" returntype="any" output="no" hint="I create a new workbook">
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="string" name="bookName" required="Yes">
		<cfset var vr = createObject("java","java.io.FileOutputStream").init(bookName)>
		<cfset workbook.write(vr)>
		<cfreturn vr>
	</cffunction>

	<cffunction name="createBookVar" returntype="any" output="no" hint="I return the workbook raw data.">
		<!--- I return the raw workbook data without creating a file so that it can be streamed directly to the browser. --->
		<cfargument type="any" name="workbook" default="wb" />
		<cfset var retVal = createObject("java", "java.io.ByteArrayOutputStream").init() />
		<cfset workbook.write(retVal) />
		<cfreturn retVal.toByteArray() />
	</cffunction>

	<cffunction name="createSheet" returntype="any" output="no" hint="I create a new sheet">
		<!---
		Sheets need to be created before the workBook creation!
		--->
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="string" name="sheetName" required="Yes">
		<cfreturn workbook.createSheet(sheetName)>
	</cffunction>

	<cffunction name="getRow" returntype="any" output="no" hint="I return an existing row.">
		<!---
		Returns the existing row. If a row doesn't exist, it creates that row.
		--->
		<cfargument type="any" name="sheet" default="sh" />
		<cfargument type="numeric" name="rownum" required="yes" />
		<cfset var theRow = "" />

		<cftry>
			<cfset theRow = arguments.sheet.getRow(arguments.rownum) />
			<cfif not isDefined("theRow") or not isObject(theRow)>
				<cfset theRow = arguments.sheet.createRow(arguments.rownum) />
			</cfif>

			<cfcatch type="any">
				<cfset theRow = arguments.sheet.createRow(arguments.rownum) />
			</cfcatch>
		</cftry>

		<cfreturn theRow />
	</cffunction>

	<cffunction name="getCell" returntype="any" output="no" hint="I return an existing cell.">
		<!---
		Returns the existing cell. If a cell doesn't exist, it creates that cell.
		--->
		<cfargument type="any" name="sheet" default="sh" />
		<cfargument type="numeric" name="rownum" required="yes" />
		<cfargument type="numeric" name="cellnum" required="yes" />
		<cfset var row = "" />
		<cfset var cell = "" />

		<cfset row = getRow(arguments.sheet, arguments.rownum) />
		<cftry>
			<cfset cell = row.getCell(arguments.cellnum) />
			<cfif not isDefined("cell") or not isObject(cell)>
				<cfset cell = row.createCell(arguments.cellnum) />
			</cfif>

			<cfcatch type="any">
				<cfset cell = row.createCell(arguments.cellnum) />
			</cfcatch>
		</cftry>

		<cfreturn cell />
	</cffunction>

	<cffunction name="createCell" returntype="any" output="no" hint="I create a cell">
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum" required="Yes">
		<cfargument type="numeric" name="cellnum" required="Yes">
		<cfset var cell = "" />

		<cfset cell = getCell(arguments.sheet, arguments.rownum, arguments.cellnum) />

		<cfreturn cell />
	</cffunction>

	<cffunction name="createRegion" returntype="any" output="no" hint="I create a cell">
		<cfargument type="numeric" name="rowFrom" required="Yes" />
		<cfargument type="numeric" name="cellFrom" required="Yes" />
		<cfargument type="numeric" name="rowTo" required="Yes" />
		<cfargument type="numeric" name="cellTo" required="Yes" />
		<cfargument type="boolean" name="isXLSX" default="false" />
		<cfset var theRegion = "" />

		<!---
			Modified to be able to make regions in xlsx workbooks.
		--->

		<cfif arguments.isXLSX>
			<cfset theRegion = createObject("java", "org.apache.poi.ss.util.CellRangeAddress").init(rowFrom, rowTo, cellFrom, cellTo)>
		<cfelse>
			<cfset theRegion = createObject("java", "org.apache.poi.hssf.util.Region").init(rowFrom, cellFrom, rowTo, cellTo) />
		</cfif>

		<cfreturn theRegion />
	</cffunction>

	<cffunction name="setCellValue" returntype="any" output="no" hint="I set a value in the given ROWxCELL">
		<!--- CENTER,CENTER_SELECTION,FILL,GENERAL,JUSTIFY,LEFT,RIGHT --->
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum" required="Yes">
		<cfargument type="numeric" name="cellnum" required="Yes">
		<cfargument name="val" default="">
		<!--- Changed to use new getCell function --->
		<cfset var vr = getCell(sheet, rownum, cellnum) />
		<cfset vr.setCellValue(val)>
		<cfreturn vr>
	</cffunction>

	<cffunction name="createDateCell" returntype="void" output="yes" hint="I create date cell">
		<!---
			"m/d/yy", "d-mmm-yy", "d-mmm", "mmm-yy", "h:mm AM/PM", "h:mm:ss AM/PM", "h:mm", "h:mm:ss"
			"m/d/yy h:mm", "mm:ss", "[h]:mm:ss", "mm:ss.0"
		--->
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum" required="Yes">
		<cfargument type="numeric" name="cellnum" required="Yes">
		<cfargument type="string" name="dtformat" default="m/d/yy h:mm">
		<cfargument type="string" name="datetime" default="#Now()#">
		<cfset var cellStyle = workbook.createCellStyle()>
		<cfset var cell = "">
		<cfset cellStyle.setDataFormat(createObject("java","org.apache.poi.hssf.usermodel.HSSFDataFormat").getBuiltinFormat(dtformat))>
		<cfset cell = getCell(arguments.sheet, arguments.rownum, arguments.cellnum) />
		<!--- <cfset cell = sheet.createRow(rownum).createCell(cellnum)> --->
		<cfset cell.setCellValue(datetime)>
		<cfset cell.setCellStyle(cellStyle)>
		<cfreturn>
	</cffunction>

	<cffunction name="setCellFormula" returntype="void" output="no" hint="I set a formula as cell value">
		<!---
		keep in mind that formulas should be exactly as they were if you were putting
		them in the cell sheet, e.g. you must reference other cells using the A1, B4 etc.
		format. Also, make sure you omit the "=" in the formula
		--->
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum" required="Yes">
		<cfargument type="numeric" name="cellnum" required="Yes">
		<cfargument name="frml" default="">
		<cfset var cell = "" />
		<cfset cell = getCell(arguments.sheet, arguments.rownum, arguments.cellnum) />
		<cfset cell.setCellFormula(frml) />
		<cfreturn>
	</cffunction>

	<cffunction name="setCellAlign" returntype="void" output="no" hint="I set the alignment for the given workbook and cell">
		<!--- CENTER,CENTER_SELECTION,FILL,GENERAL,JUSTIFY,LEFT,RIGHT --->
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum">
		<cfargument type="numeric" name="cellnum">
		<cfargument type="any" name="cell" default="cl">
		<cfargument name="align" required="No" type="string" default="left">
		<cfargument name="wrap" required="No" type="boolean" default="false">
		<cfset var cellStyle = "">
		<cfset var al = "cellStyle.ALIGN_#align#">

		<cfif not IsDefined("arguments.cell") or not isObject(arguments.cell)>
			<cfset cell = this.getCell(arguments.sheet, arguments.rownum, arguments.cellnum) />
		</cfif>
		<cfset cellStyle = workbook.createCellStyle()>
		<cfset cellStyle.setAlignment(Evaluate(al))>
		<cfset cellStyle.setWrapText(wrap)>
		<cfset cell.setCellStyle(cellStyle)>
		<cfreturn>
	</cffunction>

	<cffunction name="setCellBorder" returntype="void" output="no" hint="I set border for a given cell">
		<!--- If is defined BORDER and/or COLOR, then the other Bx/Cx parameters will not take effect --->
		<!--- Borders:
		DASH_DOT, DASH_DOT_DOT, DASHED, DOTTED, DOUBLE, HAIR, MEDIUM, MEDIUM_DASH_DOT,
		MEDIUM_DASH_DOT_DOT, MEDIUM_DASHED, NONE, SLANTED_DASH_DOT, THICK, THIN
		--->
		<!--- Colors:
		AQUA, BLACK, BLUE, BLUE_GREY, BRIGHT_GREEN, BROWN, CORAL, CORNFLOWER_BLUE, DARK_BLUE,
		DARK_GREEN, DARK_RED, DARK_TEAL, DARK_YELLOW, GOLD, GREEN, GREY_25_PERCENT, GREY_40_PERCENT,
		GREY_50_PERCENT, GREY_80_PERCENT, INDIGO, LAVENDER, LEMON_CHIFFON, LIGHT_BLUE,
		LIGHT_CORNFLOWER_BLUE, LIGHT_GREEN, LIGHT_ORANGE, LIGHT_TURQUOISE, LIGHT_YELLOW, LIME, MAROON,
		OLIVE_GREEN, ORANGE, ORCHID, PALE_BLUE, PINK, PLUM, RED, ROSE, ROYAL_BLUE, SEA_GREEN, SKY_BLUE,
		TAN, TEAL, TURQUOISE, VIOLET, WHITE, YELLOW
		--->
		<cfargument type="any" required="Yes" name="workbook" default="wb">
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum">
		<cfargument type="numeric" name="cellnum">
		<cfargument type="any" name="cell" default="cl">
		<cfargument name="border" required="No" type="Any">
		<cfargument name="color" required="No" type="Any">
		<cfargument name="btop" required="No" type="Any">
		<cfargument name="ctop" required="No" type="Any">
		<cfargument name="bbottom" required="No" type="Any">
		<cfargument name="cbottom" required="No" type="Any">
		<cfargument name="bleft" required="No" type="Any">
		<cfargument name="cleft" required="No" type="Any">
		<cfargument name="bright" required="No" type="Any">
		<cfargument name="cright" required="No" type="Any">
		<cfset var cellStyle = "">
		<cfset var al = "">
		<cfparam name="color" default="black">
		<cfif not IsDefined("arguments.cell") or not isObject(arguments.cell)>
			<!--- Modified to use getCell --->
			<cfset cell = this.getCell(arguments.sheet, arguments.rownum, arguments.cellnum) />
		</cfif>
		<cfset cellStyle = workbook.createCellStyle()>
		<cfif IsDefined("arguments.border")>
			<cfset al = "cellStyle.BORDER_#border#">
			<cfset cellStyle.setBorderBottom(Evaluate(al))>
			<cfset cellStyle.setBottomBorderColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
			<cfset cellStyle.setBorderTop(Evaluate(al))>
			<cfset cellStyle.setTopBorderColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
			<cfset cellStyle.setBorderLeft(Evaluate(al))>
			<cfset cellStyle.setLeftBorderColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
			<cfset cellStyle.setBorderRight(Evaluate(al))>
			<cfset cellStyle.setRightBorderColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
		<cfelse>
			<cfif IsDefined("arguments.btop")>
				<cfset al = "cellStyle.BORDER_#btop#">
				<cfif IsDefined("arguments.ctop")>
					<cfset color = ctop>
				</cfif>
				<cfset cellStyle.setBorderTop(Evaluate(al))>
				<cfset cellStyle.setTopBorderColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
			</cfif>
			<cfif IsDefined("arguments.bbottom")>
				<cfset al = "cellStyle.BORDER_#bbottom#">
				<cfif IsDefined("arguments.cbottom")>
					<cfset color = cbottom>
				</cfif>
				<cfset cellStyle.setBorderBottom(Evaluate(al))>
				<cfset cellStyle.setBottomBorderColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
			</cfif>
			<cfif IsDefined("arguments.bleft")>
				<cfset al = "cellStyle.BORDER_#bleft#">
				<cfif IsDefined("arguments.cleft")>
					<cfset color = cleft>
				</cfif>
				<cfset cellStyle.setBorderLeft(Evaluate(al))>
				<cfset cellStyle.setLeftBorderColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
			</cfif>
			<cfif IsDefined("arguments.bright")>
				<cfset al = "cellStyle.BORDER_#bright#">
				<cfif IsDefined("arguments.cright")>
					<cfset color = cright>
				</cfif>
				<cfset cellStyle.setBorderRight(Evaluate(al))>
				<cfset cellStyle.setRightBorderColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
			</cfif>
		</cfif>
		<cfset cell.setCellStyle(cellStyle)>
		<cfreturn>
	</cffunction>

	<cffunction name="setCellPattern" returntype="void" output="no" hint="I set the pattern for a given cell">
		<!---
		NO_FILL, SOLID_FOREGROUND, FINE_DOTS, ALT_BARS, SPARSE_DOTS, THICK_HORZ_BANDS,
		THICK_VERT_BANDS, THICK_BACKWARD_DIAG, THICK_FORWARD_DIAG, BIG_SPOTS, BRICKS, THIN_HORZ_BANDS,
		THIN_VERT_BANDS, THIN_BACKWARD_DIAG, THIN_FORWARD_DIAG, SQUARES, DIAMONDS
		--->
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum">
		<cfargument type="numeric" name="cellnum">
		<cfargument type="any" name="cell" default="cl">
		<cfargument name="pattern" required="No" type="Any" default="NO_FILL">
		<cfargument name="color" required="No" type="Any" default="white">
		<cfset var cellStyle = "">
		<cfset var al = "cellStyle.#pattern#">
		<cfif not IsDefined("arguments.cell") or not isObject(arguments.cell)>
			<!--- Modified to use getCell --->
			<cfset cell = this.getCell(arguments.sheet, arguments.rownum, arguments.cellnum) />
		</cfif>
		<cfset cellStyle = workbook.createCellStyle()>
		<cfset cellStyle.setFillPattern(Evaluate(al))>
		<cfset cellStyle.setFillBackgroundColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex())>
		<cfset cell.setCellStyle(cellStyle)>

		<cfset cell.setCellStyle(cellStyle)>
		<cfreturn>
	</cffunction>

	<cffunction name="mergeRegion" returntype="void" output="no" hint="I merge a region">
		<cfargument type="any" name="sheet" default="sh" />
		<cfargument type="numeric" name="rowFrom" required="Yes" />
		<cfargument type="numeric" name="cellFrom" required="Yes" />
		<cfargument type="numeric" name="rowTo" required="Yes" />
		<cfargument type="numeric" name="cellTo" required="Yes" />

		<!--- Modified to use new addMergedRegion and passes in the new field --->

		<cfset var isXLSX = yesNoFormat(findNoCase("XSSF", arguments.sheet.class.getCanonicalName())) />
		<cfset sheet.addMergedRegion(
					createRegion(
						rowFrom,
						cellFrom,
						rowTo,
						arguments.cellTo,
						isXLSX)) />
		<cfreturn />
	</cffunction>

	<cffunction name="setCellFontStyle" returntype="void" output="no" hint="I set the font style for a given cell">
		<!--- typeOffset values: sub, sup --->
		<!--- underline values: NONE, SINGLE, DOUBLE, SINGLE_ACCOUNTING, DOUBLE_ACCOUNTING --->
		<cfargument type="any" name="workbook" default="wb" required="Yes" />
		<cfargument type="any" name="sheet" default="sh" />
		<cfargument type="numeric" name="rownum" />
		<cfargument type="numeric" name="cellnum" />
		<cfargument type="any" name="cell" default="cl" />
		<cfargument name="fontHeight" required="No" type="numeric" />
		<cfargument name="fontName" required="No" type="string" />
		<cfargument name="italic" required="No" type="boolean" default="false" />
		<cfargument name="strikeout" required="No" type="boolean" default="false" />
		<cfargument name="color" required="No" type="string" default="black" />
		<cfargument name="bold" required="No" type="boolean" default="false" />
		<cfargument name="typeOffset" required="No" type="string" default="0" />
		<cfargument name="underline" required="No" type="string" default="none" />
		<cfif not IsDefined("arguments.cell") or not isObject(arguments.cell)>
			<!--- Modified to use the new getCell --->
			<cfset cell = this.getCell(arguments.sheet, arguments.rownum, arguments.cellnum) />
		</cfif>
		<cfset font = workbook.createFont() />
		<cfif IsDefined("arguments.fontHeight")>
			<cfset font.setFontHeightInPoints(fontHeight) />
		</cfif>
		<cfif IsDefined("arguments.fontName")>
			<cfset font.setFontName(fontName) />
		</cfif>
		<cfset font.setItalic(italic) />
		<cfset font.setStrikeout(strikeout) />
		<cfset font.setColor(createObject("java","org.apache.poi.hssf.util.HSSFColor$#UCase(color)#").getIndex()) />
		<cfif bold>
			<cfset font.setBoldweight(4796) />
		</cfif>
		<cfif typeOffset eq "sup">
			<cfset font.setTypeOffset(1) />
		<cfelse>
			<cfif typeOffset eq "sub">
				<cfset font.setTypeOffset(2) />
			</cfif>
		</cfif>
		<cfswitch expression="#underline#">
			<cfcase value="none">
				<cfset underline = 0 />
			</cfcase>
			<cfcase value="SINGLE">
				<cfset underline = 1 />
			</cfcase>
			<cfcase value="DOUBLE">
				<cfset underline = 2 />
			</cfcase>
			<cfcase value="SINGLE_ACCOUNTING">
				<cfset underline = 33 />
			</cfcase>
			<cfcase value="DOUBLE_ACCOUNTING">
				<cfset underline = 34 />
			</cfcase>
		</cfswitch>
		<cfset font.setUnderline(underline) />
		<cfset style = workbook.createCellStyle() />
		<cfset style.setFont(font) />
		<cfset cell.setCellStyle(style) />
		<cfreturn />
	</cffunction>

	<cffunction name="readBook" returntype="any" output="yes" hint="I read an existing workbook">
		<cfargument type="string" name="bookName" required="Yes">
		<cfset var vr = "">
		<cftry>
			<cfset fileIn = createObject("java","java.io.FileInputStream").init(bookName)>
			<cfset fs = createObject("java","org.apache.poi.poifs.filesystem.POIFSFileSystem").init(fileIn)>
			<cfset vr = createObject("java","org.apache.poi.hssf.usermodel.HSSFWorkbook").init(fs)>
			<cfreturn vr>
			<cfcatch type="Any">
				No such workbook
			</cfcatch>
		</cftry>
	</cffunction>

	<cffunction name="writeBook" returntype="void" output="yes" hint="I write into an existing workbook">
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="string" name="bookName" required="no">
		<cftry>
			<cfset createObject("java","java.io.FileOutputStream").init(bookName)>
			<cfset workbook.write(fileOut)>
			<cfcatch type="Any">
				No such workbook
			</cfcatch>
		</cftry>
		<cfreturn>
	</cffunction>

	<cffunction name="readSheet" returntype="any" output="yes" hint="I read an existing sheet by name or by number">
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="string" name="sheetName" required="no" default="">
		<cfargument type="numeric" name="sheetNumber" required="no" default="0">
		<cfset vr = " ">
		<cftry>
			<cfif Len(Trim(sheetName))>
				<cfset vr = workbook.getSheet(sheetName)>
				<cfreturn vr>
			<cfelse>
				<cfset vr = workbook.getSheetAt(sheetNumber)>
				<cfreturn vr>
			</cfif>
			<cfcatch type="Any">
				No such sheet
			</cfcatch>
		</cftry>
	</cffunction>

	<cffunction name="getCellValue" returntype="any" output="no" hint="I get the value from the given ROWxCELL">
		<!--- CENTER,CENTER_SELECTION,FILL,GENERAL,JUSTIFY,LEFT,RIGHT --->
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum" required="Yes">
		<cfargument type="numeric" name="cellnum" required="Yes">
		<cfset var row = sheet.getRow(rownum)>
		<cfset var cell = row.getCell(cellnum)>
		<cfif IsDefined("cell")>
			<cftry>
				<cfswitch expression="#cell.getCellType()#">
					<cfcase value="0">
						<cftry>
							<cfset ut = createObject("java","org.apache.poi.hssf.usermodel.HSSFDateUtil")>
							<cfif ut.isCellDateFormatted(cell)>
								<cfset vr = cell.getDateCellValue()>
							<cfelse>
								<cfset vr = cell.getNumericCellValue()>
							</cfif>
							<cfcatch type="Any">
								<cfset vr = cell.getNumericCellValue()>
							</cfcatch>
						</cftry>
					</cfcase>
					<cfcase value="1">
						<cfset vr = cell.getStringCellValue()>
					</cfcase>
					<cfcase value="2">
						<cfset vr = cell.getCellFormula()>
					</cfcase>
					<cfcase value="3">
						<cfset vr = "">
					</cfcase>
					<cfcase value="4">
						<cfset vr = cell.getBooleanCellValue()>
					</cfcase>
					<cfcase value="5">
						<cfset vr = cell.getErrorCellValue()>
					</cfcase>
				</cfswitch>
				<cfcatch type="Any">
					<cfset vr = "">
				</cfcatch>
			</cftry>
		<cfelse>
			<cfset vr = "">
		</cfif>
		<cfreturn vr>
	</cffunction>

	<cffunction name="getCellType" returntype="any" output="no" hint="I get the type of the given ROWxCELL">
		<!--- CENTER,CENTER_SELECTION,FILL,GENERAL,JUSTIFY,LEFT,RIGHT --->
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum" required="Yes">
		<cfargument type="numeric" name="cellnum" required="Yes">
		<cfset var row = sheet.getRow(rownum)>
		<cfset var cell = row.getCell(cellnum)>
		<cftry>
			<cfswitch expression="#cell.getCellType()#">
				<cfcase value="0">
					<cftry>
						<cfset ut = createObject("java","org.apache.poi.hssf.usermodel.HSSFDateUtil")>
						<cfif ut.isCellDateFormatted(cell)>
							<cfset vr = "datetime">
						<cfelse>
							<cfset vr = "numeric">
						</cfif>
						<cfcatch type="Any">
							<cfset vr = "numeric">
						</cfcatch>
					</cftry>
				</cfcase>
				<cfcase value="1">
					<cfset vr = "string">
				</cfcase>
				<cfcase value="2">
					<cfset vr = "formula">
				</cfcase>
				<cfcase value="3">
					<cfset vr = "blank">
				</cfcase>
				<cfcase value="4">
					<cfset vr = "boolean">
				</cfcase>
				<cfcase value="5">
					<cfset vr = "error">
				</cfcase>
			</cfswitch>
			<cfcatch type="Any">
				<cfset vr = "error">
			</cfcatch>
		</cftry>
		<cfreturn vr>
	</cffunction>

	<cffunction name="setRowHeight" returntype="void" output="yes" hint="I set row height">
		<!--- row height is in twips (1/20th of a pixel) --->
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum" required="no" default="0">
		<cfargument type="numeric" name="height" required="no" default="100">
		<cfset var row = "" />
		<cfset row = getRow(arguments.sheet, arguments.rownum) />
		<cfset row.setHeight(arguments.height)>
		<cfreturn>
	</cffunction>

	<cffunction name="setCellWidth" returntype="void" output="yes" hint="I set cell width">
		<!--- set the column width (in units of 1/256th of a character width)  --->
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="cellnum" required="no" default="0">
		<cfargument type="numeric" name="width" required="no" default="100">
		<cfset sheet.setColumnWidth(cellnum, width)>
		<cfreturn>
	</cffunction>

	<cffunction name="setCellDataFormat" returntype="void" output="yes" hint="I set the data format for a given cell">
		<!---
		"General", "0", "0.00", "#,##0", "#,##0.00", "($#,##0_);($#,##0)", "($#,##0_);[Red]($#,##0)"
		"($#,##0.00);($#,##0.00)", "($#,##0.00_);[Red]($#,##0.00)", "0%", "0.00%", "0.00E+00"
		"# ?/?", "# ??/??", "m/d/yy", "d-mmm-yy", "d-mmm", "mmm-yy", "h:mm AM/PM", "h:mm:ss AM/PM"
		"h:mm", "h:mm:ss", "m/d/yy h:mm", "(#,##0_);[Red](#,##0)", "(#,##0.00_);(#,##0.00)"
		"(#,##0.00_);[Red](#,##0.00)", "_(*#,##0_);_(*(#,##0);_(* \"-\"_);_(@_)"
		"_($*#,##0_);_($*(#,##0);_($* \"-\"_);_(@_)", "_(*#,##0.00_);_(*(#,##0.00);_(*\"-\"??_);_(@_)"
		"_($*#,##0.00_);_($*(#,##0.00);_($*\"-\"??_);_(@_)", "mm:ss" , "[h]:mm:ss", "mm:ss.0"
		"##0.0E+0", "@" - This is text format., "text" - Alias for "@"
		--->
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="rownum" required="Yes">
		<cfargument type="numeric" name="cellnum" required="Yes">
		<cfargument type="string" name="format" required="no" default="general">
		<cfargument type="any" name="val">
		<cfset var frmt = workbook.createDataFormat()>
		<cfset var style = "">
		<cfset var cell = this.getCell(arguments.sheet, arguments.rownum, arguments.cellnum) />
		<cfif IsDefined("arguments.val")>
			<cfset cell.setCellValue(val)>
		</cfif>
		<cfset style = workbook.createCellStyle()>
		<cfset style.setDataFormat(frmt.getFormat(format))>
		<cfset cell.setCellStyle(style)>
		<cfreturn>
	</cffunction>

	<cffunction name="fitSheet" returntype="void" output="yes" hint="I fit the provided sheet">
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="numeric" name="hpages" default="1">
		<cfargument type="numeric" name="wpages" default="1">
		<cfargument type="boolean" name="autobreaks" default="true"> <!--- show automatic page breaks or not --->
		<cfset var ps= sheet.getPrintSetup()>
		<cfset sheet.setAutobreaks(autobreaks)>
		<cfset ps.setFitHeight(hpages)>
		<cfset ps.setFitWidth(wpages)>
		<cfreturn>
	</cffunction>

	<cffunction name="setPrintArea" returntype="void" output="yes" hint="I set print area">
		<cfargument type="any" name="workbook" default="wb">
		<cfargument type="numeric" name="sheetnum" default="0">
		<cfargument type="numeric" name="rowFrom" required="Yes">
		<cfargument type="numeric" name="cellFrom" required="Yes">
		<cfargument type="numeric" name="rowTo" required="Yes">
		<cfargument type="numeric" name="cellTo" required="Yes">
		<cfset workbook.setPrintArea(sheetnum, rowFrom, cellFrom, rowTo, cellTo)>
		<cfreturn>
	</cffunction>

	<cffunction name="setPageNum" returntype="void" output="yes" hint="I set page number">
		<cfargument type="any" name="sheet" default="sh">
		<cfset var footer = sheet.getFooter()>
		<cfset footer.setRight( "Page #footer.page()# of #footer.numPages()#")>
		<cfreturn>
	</cffunction>

	<cffunction name="setHeaders" returntype="void" output="yes" hint="I set page headers">
		<!--- italic, bold --->
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="string" name="lheader"> <!--- left header --->
		<cfargument type="string" name="cheader"> <!--- center header --->
		<cfargument type="string" name="rheader"> <!--- right header --->
		<cfargument type="string" name="lheaderfont"> <!--- left header font --->
		<cfargument type="string" name="lheaderfontstyle" default="normal"> <!--- left header font style --->
		<cfargument type="numeric" name="lheaderfontsize"> <!--- left header font size --->
		<cfargument type="string" name="cheaderfont"> <!--- center header font --->
		<cfargument type="string" name="cheaderfontstyle" default="normal"> <!--- center header font style --->
		<cfargument type="numeric" name="cheaderfontsize"> <!--- center header font size --->
		<cfargument type="string" name="rheaderfont"> <!--- right header font --->
		<cfargument type="string" name="rheaderfontstyle" default="normal"> <!--- right header font style --->
		<cfargument type="numeric" name="rheaderfontsize"> <!--- right header font size --->
		<cfset var hdr = "">
		<cfset var header = sheet.getHeader()>
		<cfif IsDefined("arguments.lheader")>
			<cfif IsDefined("arguments.lheaderfont")>
				<cfset hdr = hdr & header.font(lheaderfont, lheaderfontstyle)>
			</cfif>
			<cfif IsDefined("arguments.lheaderfontsize")>
				<cfset hdr = hdr & header.fontSize(lheaderfontsize)>
			</cfif>
			<cfset hdr = hdr & lheader>
			<cfset header.setLeft(hdr)>
		</cfif>
		<cfif IsDefined("arguments.cheader")>
			<cfset hdr = "">
			<cfif IsDefined("arguments.cheaderfont")>
				<cfset hdr = hdr & header.font(cheaderfont, cheaderfontstyle)>
			</cfif>
			<cfif IsDefined("arguments.cheaderfontsize")>
				<cfset hdr = hdr & header.fontSize(cheaderfontsize)>
			</cfif>
			<cfset hdr = hdr & cheader>
			<cfset header.setCenter(hdr)>
		</cfif>
		<cfif IsDefined("arguments.rheader")>
			<cfset hdr = "">
			<cfif IsDefined("arguments.rheaderfont")>
				<cfset hdr = hdr & header.font(rheaderfont, rheaderfontstyle)>
			</cfif>
			<cfif IsDefined("arguments.rheaderfontsize")>
				<cfset hdr = hdr & header.fontSize(rheaderfontsize)>
			</cfif>
			<cfset hdr = hdr & rheader>
			<cfset header.setRight(hdr)>
		</cfif>
		<cfreturn>
	</cffunction>

	<cffunction name="setFooters" returntype="void" output="yes" hint="I set page footers">
		<!--- italic, bold --->
		<cfargument type="any" name="sheet" default="sh">
		<cfargument type="string" name="lfooter"> <!--- left footer --->
		<cfargument type="string" name="cfooter"> <!--- center footer --->
		<cfargument type="string" name="rfooter"> <!--- right footer --->
		<cfargument type="string" name="lfooterfont"> <!--- left footer font --->
		<cfargument type="string" name="lfooterfontstyle" default="normal"> <!--- left footer font style --->
		<cfargument type="numeric" name="lfooterfontsize"> <!--- left footer font size --->
		<cfargument type="string" name="cfooterfont"> <!--- center footer font --->
		<cfargument type="string" name="cfooterfontstyle" default="normal"> <!--- center footer font style --->
		<cfargument type="numeric" name="cfooterfontsize"> <!--- center footer font size --->
		<cfargument type="string" name="rfooterfont"> <!--- right footer font --->
		<cfargument type="string" name="rfooterfontstyle" default="normal"> <!--- right footer font style --->
		<cfargument type="numeric" name="rfooterfontsize"> <!--- right footer font size --->
		<cfset var hdr = "">
		<cfset var footer = sheet.getFooter()>
		<cfif IsDefined("arguments.lfooter")>
			<cfif IsDefined("arguments.lfooterfont")>
				<cfset hdr = hdr & footer.font(lfooterfont, lfooterfontstyle)>
			</cfif>
			<cfif IsDefined("arguments.lfooterfontsize")>
				<cfset hdr = hdr & footer.fontSize(lfooterfontsize)>
			</cfif>
			<cfset hdr = hdr & lfooter>
			<cfset footer.setLeft(hdr)>
		</cfif>
		<cfif IsDefined("arguments.cfooter")>
			<cfset hdr = "">
			<cfif IsDefined("arguments.cfooterfont")>
				<cfset hdr = hdr & footer.font(cfooterfont, cfooterfontstyle)>
			</cfif>
			<cfif IsDefined("arguments.cfooterfontsize")>
				<cfset hdr = hdr & footer.fontSize(cfooterfontsize)>
			</cfif>
			<cfset hdr = hdr & cfooter>
			<cfset footer.setCenter(hdr)>
		</cfif>
		<cfif IsDefined("arguments.rfooter")>
			<cfset hdr = "">
			<cfif IsDefined("arguments.rfooterfont")>
				<cfset hdr = hdr & footer.font(rfooterfont, rfooterfontstyle)>
			</cfif>
			<cfif IsDefined("arguments.rfooterfontsize")>
				<cfset hdr = hdr & footer.fontSize(rfooterfontsize)>
			</cfif>
			<cfset hdr = hdr & rfooter>
			<cfset footer.setRight(hdr)>
		</cfif>
		<cfreturn>
	</cffunction>

	<cffunction name="easyQuery2Excel" returntype="void" hint="I create an Excel file from a given query">
		<cfargument type="string" name="bookName" required="Yes">
		<cfargument type="string" name="sheetName" default="Sheet 1">
		<cfargument type="string" name="titles"> <!--- comma separated list of column names --->
		<cfargument type="query" name="data" required="Yes">
		<cfset var wb = this.init()>
		<cfset var sh = this.createSheet(wb, sheetName)>
		<cfset var cell = "">
		<cfif IsDefined("arguments.titles")>
			<cfloop index="i" from="1" to="#ListLen(titles, ',')#">
				<cfset cell = this.setCellValue(sh, 0, javacast("int",i-1), ListGetAt(titles, i, ","))>
				<cfset this.setCellAlign(workbook=wb, cell=cell, align="center")>
				<cfset this.setCellBorder(workbook=wb, cell=cell, bbottom="THICK", cbottom="black")>
				<cfset this.setCellFontStyle(workbook=wb, cell=cell, bold="true")>
			</cfloop>
		</cfif>
		<cfloop query="data">
			<cfloop index="i" from="1" to="#ListLen(columnList, ',')#">
				<cfset cell = this.setCellValue(sh, currentrow, javacast("int",i-1), javacast("string", Evaluate(ListGetAt(columnList, i, ","))))>
			</cfloop>
		</cfloop>
		<cfset this.close(this.createBook(wb, bookName))>
		<cfreturn>
	</cffunction>

	<cffunction name="worksheet2array" returntype="struct" hint="I create an array from a worksheet">
		<cfargument type="string" name="bookName" required="Yes">
		<cfargument type="string" name="sheetName" default="Sheet 1">

		<cfset var wb = this.readBook(bookName)>
		<cfset var sh = this.readSheet(wb, sheetName)>
		<cfset var cv = "">
		<cfset var lastCol = sh.getRow(0).getLastCellNum() - 1>
		<cfset var lastRow = "">
		<cfset var rez = StructNew()>

		<cfset lastRow = sh.getLastRowNum()>
		<cfset arr = ArrayNew(2)>
		<cfloop index="i" from="0" to="#lastRow#">
			<cfloop index="j" from="0" to="#lastCol#">
				<cfset vr = this.getCellValue(sh, javacast("int",i), javacast("int",j))>
				<cfset arr[i+1][j+1] = vr>
			</cfloop>
		</cfloop>
		<cfset rez.lastCol = lastCol + 1>
		<cfset rez.lastRow = lastRow + 1>
		<cfset rez.arr = arr>
		<!---
		<cfset this.close(wb)>
		<cfset this.close(this.createBook(wb, bookName))>
		--->
		<cfreturn rez>
	</cffunction>

	<cffunction name="addJPGImage" returntype="any" hint="I put a picture in the excel document.">
		<cfargument type="any" name="workbook" default="wb" />
		<cfargument type="any" name="sheet" default="sh" />
		<cfargument type="numeric" name="rownum" required="true" />
		<cfargument type="numeric" name="cellnum" required="true" />
		<cfargument type="any" name="picData" required="true" />

		<!---
			This function calls the addImage function to put a JPEG image
			into the workbook.
		--->

		<cftry>
			<cfset this.addImage(
					arguments.workbook,
					arguments.sheet,
					arguments.rownum,
					arguments.cellnum,
					arguments.picData,
					arguments.workbook.PICTURE_TYPE_JPEG) />

			<cfcatch type="any">
				<cfset this.setCellValue(
					arguments.sheet,
					arguments.rownum,
					arguments.cellnum,
					"AddJPGImage does not work with this version of POI.") />
			</cfcatch>
		</cftry>

		<cfreturn />
	</cffunction>

	<cffunction name="addPNGImage" returntype="any" hint="I put a picture in the excel document.">
		<cfargument type="any" name="workbook" default="wb" />
		<cfargument type="any" name="sheet" default="sh" />
		<cfargument type="numeric" name="rownum" required="true" />
		<cfargument type="numeric" name="cellnum" required="true" />
		<cfargument type="any" name="picData" required="true" />

		<!---
			This function calls the addImage function to put a PNG image
			into the workbook.
		--->

		<cftry>
			<cfset this.addImage(
					arguments.workbook,
					arguments.sheet,
					arguments.rownum,
					arguments.cellnum,
					arguments.picData,
					arguments.workbook.PICTURE_TYPE_PNG) />

			<cfcatch type="any">
				<cfset this.setCellValue(
					arguments.sheet,
					arguments.rownum,
					arguments.cellnum,
					"AddPNGImage does not work with this version of POI.") />
			</cfcatch>
		</cftry>

		<cfreturn />
	</cffunction>

	<cffunction name="addImage" returntype="any"
		hint="I put a picture in the excel document">
		<cfargument type="any" name="workbook" default="wb" />
		<cfargument type="any" name="sheet" default="sh" />
		<cfargument type="numeric" name="rownum" required="true" />
		<cfargument type="numeric" name="cellnum" required="true" />
		<cfargument type="any" name="picData" required="true" />
		<cfargument type="any" name="picType" required="true" />

		<!---
			This function places an image into the workbook or it
			displays an error message if the version of POI does not
			support images.
			When used with cfchart, charts can be placed into excel
			spreadsheets as image.
		--->

		<cfset var pictureIdx = "" />
		<cfset var helper = "" />
		<cfset var drawing = "" />
		<cfset var anchor = "" />
		<cfset var pict = "" />

		<cftry>
			<cfset pictureIdx = arguments.workbook.addPicture(
					arguments.picData,
					arguments.picType) />
			<cfset helper = arguments.workbook.getCreationHelper() />
			<cfset drawing = arguments.sheet.createDrawingPatriarch() />
			<cfset anchor = helper.createClientAnchor() />

			<cfset anchor.setCol1(arguments.cellnum) />
			<cfset anchor.setRow1(arguments.rownum) />
			<cfset pict = drawing.createPicture(anchor, pictureIdx) />
			<cfset pict.resize() />

			<cfcatch type="any">
				<cfset this.setCellValue(
					arguments.sheet,
					arguments.rownum,
					arguments.cellnum,
					"AddImage does not work with this version of POI.") />
			</cfcatch>
		</cftry>

		<cfreturn />
	</cffunction>
</cfcomponent>

